// Approximated estimation with regression model
// - simulade data with 2nd order reg. model
// Task one 
// - estimate the data but with 1st order reg. model
//   you obtain (i) estimated parameters, (ii) prediction
// Task two
// - estimate the same data sample, however, with 2n order reg. model
// - again, you obtain (i) estimated parameters, (ii) prediction
// Notice: the parametrs differ (for 2n order they resemble simulated),
//         however, the predictions are simular (the 1st order 
//         approximates the 2nd oreder one).
exec("ScIntro.sce",-1),mode(0)

nd=100;
a1=.6;  a2=.1;  r=.1;
b0=1;  b1=-1.5;  b2=.2;

// SIMULATION (second order reg. model)
y=zeros(1,nd);
u=1+rand(1,nd,'norm');                  // input
th=[b0 a1 b1 a2 b2];
for t=3:nd
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2)]; // regression vector
  y(t)=ps*th'+sqrt(r)*rand(1,1,'norm');  // output
end

// ESTIMATION (first order model)
V=zeros(4,4);
for t=2:nd
  Ps=[y(t) u(t) y(t-1) u(t-1)];         // extended regression vector
  V=V+Ps'*Ps;                           // update of statistics
end
Vy=V(1,1);                              // partitioning of
Vyp=V(2:$,1);                           //     information
Vp=V(2:$,2:$);                          //     matrix
Eth1=inv(Vp)*Vyp                        // point estimates of reg. koef
Er1=(Vy-Vyp'*inv(Vp)*Vyp)/nd            //  -"-   of norise variance

// PREDICTION (simulation with estimated parameters)
yp1=zeros(1,nd);
for t=2:nd
  ps=[u(t) yp1(t-1) u(t-1)];            // regression vector
  yp1(t)=ps*Eth1+sqrt(Er1)*rand(1,1,'norm'); // output from estimated pars
end

// RESULTS
e1=y-yp1;                               // prediction error
PE1=sqrt(e1*e1')/nd                     // sum of squares of pred. err.

subplot(121)
plot(1:nd,y,'.:',1:nd,yp1,'r.:')
//set(gcf(),'position',[70 100 600 500])
title('Simulation with 1st order model')
legend('data','simulation');

// ESTIMATION (second order model)
// - for the same simulated data we use 2nd order
//   model for estimation
V=zeros(6,6);
for t=3:nd
  Ps=[y(t) u(t) y(t-1) u(t-1) y(t-2) u(t-2)];  // ext. regression vector
  V=V+Ps'*Ps;                           // update of statistics
end
Vy=V(1,1);                              // partitioning of V
Vyp=V(2:$,1);
Vp=V(2:$,2:$);
Eth2=inv(Vp)*Vyp                        // point estimates
Er2=(Vy-Vyp'*inv(Vp)*Vyp)/nd

// PREDICTION (simulation with estimated parameters)
yp2=zeros(1,nd);
for t=3:nd
  ps=[u(t) yp2(t-1) u(t-1) yp2(t-2) u(t-2)]; // regression vector
  yp2(t)=ps*Eth2+sqrt(Er2)*rand(1,1,'norm'); // prediction
end

// RESULTS
e2=y-yp2;
PE2=sqrt(e2*e2')/nd                     // sum of squares of pred. err.
  
subplot(122)
plot(1:nd,y,'.:',1:nd,yp2,'r.:')
set(gcf(),'position',[100 100 600 500])
title('Simulation with 2nd order model')
legend('data','simulation');
