// Approximated estimation with regression model
// - simulade data with 2nd order reg. model
// Task one 
// - estimate the data but with 1st order reg. model
//   you obtain (i) estimated parameters, (ii) prediction
// Task two
// - estimate the same data sample, however, with 2n order reg. model
// - again, you obtain (i) estimated parameters, (ii) prediction
// Notice: the parametrs differ (for 2n order they resemble simulated),
//         however, the predictions are simular (the 1st order 
//         approximates the 2nd oreder one).
exec("ScIntro.sce",-1),mode(0)

nd=100;
a1=.6;  a2=.1;  r=.1;
b0=1;  b1=-1.5;  b2=.2;

// SIMULATION (second order reg. model)
y=zeros(1,nd);
u=1+rand(1,nd,'norm');                  // input
th=[b0 a1 b1 a2 b2];
for t=3:nd
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2)]; // regression vector
  y(t)=ps*th'+sqrt(r)*rand(1,1,'norm');  // output
end

// ESTIMATION
for t=3:nd
  Y(t,1)=y(t);
  X(t,:)=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1];
end
Eth=inv(X'*X)*X'*Y;
yp=X*Eth;
ep=y'-yp;
Er=ep'*ep/nd;

// PREDICTION (simulation with estimated parameters)
yp1=zeros(1,nd);
for t=3:nd
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1];  // regression vector
  yp1(t)=ps*Eth+sqrt(Er)*rand(1,1,'norm'); // output from estimated pars
end

// RESULTS
disp(Eth,'estimated parameters')
disp(Er,'estimated noise variance')

set(scf(),'position',[500 100 800 600]);
plot(1:nd,y,'.:',1:nd,yp1,'r.:')
title('Simulation with 1st order model')
legend('data','simulation');

