// Minimum variance control
// - control with inverted model 
// - on-line estimation
// -----------------------------
exec SCIHOME/ScIntro.sce, mode(0)

nd=200;
b0=1; a1=.6; b1=.2; k=2;                     // parameters for simul.
u=zeros(1,nd);
y=zeros(1,nd);
y(1)=-5;                                     // initial y
yr=sin(8*(1:nd)/nd);                         // setpoint
b0E=1; a1E=-.96; b1E=.2; kE=2; sdE=.1;       // init. pars. for estim.
V=.0001*eye(5,5);                            // init. inf. matrix

// Time loop
ctrl=1;
for t=2:nd
  
  //  control
  if ctrl==1
    u(t)=1/b0E*(yr(t)-a1E*y(t-1)-b1E*u(t-1)-kE);
  else u(t)=0; end
  
  // simulation
  y(t)=b0*u(t)+a1*y(t-1)+b1*u(t-1)+k+.1*rand(1,1,'n');

  // estimation  
  Ps=[y(t) u(t) y(t-1) u(t-1) 1]';           // extended reg. vector
  V=V+Ps*Ps';                                // statistics update
  Vy=V(1,1); Vyp=V(2:5,1); Vp=V(2:5,2:5);    // partitioning of V
  thE=inv(Vp)*Vyp;                           // reg. coef. pt. estimates
  rE=(Vy-Vyp'*thE)/t; sE=sqrt(rE);           // variance estimate
  b0E=thE(1); a1E=thE(2); b1E=thE(3); kE=thE(4); // assign the estimates
end

// Results
s=2:nd;
scf(1);
plot(s,y(s),s,u(s),s,yr(s));
legend('y','u','yr');
set(gca(),'grid',[1 1])
