// ADAPTIVE CONTROL WITH DYNAMIC REG. MODEL WITH CONSTANT
//   multivariate input and output
//   control with setpoint
//   penalization of input increments
//   receding horizon in control
// ------------------------------------------------
exec SCIHOME/ScIntro.sce, 
mode(0), getd _func

// SETTING THE TASK
ni=50;              // length of prior estimation
nd=200;             // length of control
ord=1;              // model order
pen=1;              // penalization of u  1=increments, 0=absolute u
                    //                    increm. only for ord>1
nh=3;               // length of control interval
om=1; la=.005;       // penalty:  y(t)^2  and  (u(t)-u(t-1))^2 

ada=0;              // adaptive control  1=yes, 0=no,
                    //                    else fix prior est.
typU=3;             // type of input 1=noise, 2=sin, 3=jumps

a=list(); b=list();
// full structure assumed: 
a(1)=[.3 -.2; -.05 .6];    // parameters at y
a(2)=[.1 -.1; .02 -.1];
b(1)=[1 -.4; 1.3 -.4];      // parameters at u
b(2)=[.1 0;0 .2];
b(3)=[.1 0;0 .1];
k=[1; -1];                  // model constant
sd=.05;                     // noise standard deviation

ny=size(a(1),1);  // No of values of y
nu=size(b(1),2);  // and of u

th=b(1);
for i=1:ord         // parameters -> vector th
  th=[th a(i) b(i+1)];
end
th=[th k];
                    //conversion to state model
[XX,XX,XX,nx,ny,nu]=reg2st(th,ord);

                    // penalization matrices
Om=zeros(nx,nx); 
Om(1:ny,1:ny)=om*eye(ny,ny);  // output
nn=ny+nu;
for i=1:nu                    // input increment
  Om((ny+i),(ny+i))=la;
  if (ord>1)& (pen==1) 
  Om((ny+i+nn),(ny+i+nn))=la;
  Om((ny+i),(ny+i+nn))=-la;  
  Om((ny+i+nn),(ny+i))=-la;
  end
end

// setpoint generation
g=genstp(ny,nd,[3;5],[.95;.92],[-5;10]);

// PRIOR ESTIMATION
yi=zeros(ny,ni);
select typU
case 1, ui=rand(nu,ni,'norm');
case 2, ui=[sin(20*(1:ni)/ni);cos(20*(1:ni)/ni)];
case 3, ui=sign([sin(20*(1:ni)/ni);cos(20*(1:ni)/ni)]);
end
V=zeros(nx+ny+nu,nx+ny+nu);
for i=(ord+1):ni
  ps=genps(ord,i,yi,ui);
  yi(:,i)=th*ps + sd*rand(ny,1,'norm');
  Ps=[yi(:,i); ps];
  V=V+Ps*Ps';
end
theta=v2thN(V/ni,ny);

//  TIME LOOP for adaptive control ==
S=list();
y=zeros(ny,nd); y(:,1:2)=[10 10;-10 -10];
u=zeros(nu,nd);
for t=(ord+2):(nd-nh)
  // estimation
  if ada==1
    ps=genps(ord,t-1,y,u);
    Ps=[y(:,t-1); ps];
    V=V+Ps*Ps';
    theta=v2thN(V/(t+ni),ny); // current estimates
  elseif ada==0
    theta=th';                // known parameters
  else     
    // prior pt.estimates are used
  end
  [M,N,A,nx,ny,nu]=reg2st(theta',ord);
  // generation of control law
  R=.00001*eye(nx,nx);        // regularization
  for i=nh:-1:1
    M(1:ny,$)=-g(:,t+i)+k;
    T=R+Om;
    R=inv(inv(T)+N*inv(om)*N');
  end
  // generation of optimal control
  S=inv(N'*T*N)*N'*T*M;
  x=genph(ord,t,y,u); 
  u(:,t)=-S*x;
  y(:,t)=th*[u(:,t); x]+sd*rand(ny,1,'norm');
end

// RESULTS
s=1:(nd-nh);
subplot(211)
plot(s,y(1,s),'b',s,u(1,s),'g--',s,g(1,s),'m.','markersize',2)
legend('output','input','setpoint',4);

subplot(212)
plot(s,y(2,s),'b',s,u(2,s),'g--',s,g(2,s),'m.','markersize',2)
set(gcf(),'position',[500 50 800 600])

legend('output','input','setpoint',1);
disp 'Comparison of simulated and estimated parametrs'
printf '\n  - first two rows are simulated parameters'
disp ' - second two rows are estimated parameters'
disp([th;theta'])
